unit GeoWhois;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, UrlMon;

const
  WM_DomainStatus = wm_User + 1;

type
  TZoomFactor = (a15Degrees, a30Degrees, a60Degrees, a120Degrees);

  TProjection = (Rectangular, Elliptical);

// TNetDialog ------------------------------------------------------------------
  TNetDialog = class(TForm)
    statBar: TStatusBar;
    Panel2: TPanel;
    imgMap: TImage;
    procedure FormShow(Sender: TObject);
  private
    GifFName,
    TempFName,
    MapRequest  : String;
    PostResults : TStringList;
    function GetURL(Source, Dest: string): Boolean;
    function ParseURLLocation(FName : String) : Boolean;
    procedure OnError(Error : Integer);
  public
    aZoomFactor : TZoomFactor;
    aProjection : TProjection;
    CompHandle  : HWnd;
    aDomain     : String;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetInfo;
  end;

// TGeoWhois ------------------------------------------------------------------

  TOnDomainStatus = procedure (Sender  : TObject;
                               aError  : Integer;
                               aDomain : String) of object;

  TGeoWhois = class(TComponent)
  private
    fZoomFactor     : TZoomFactor;
    fProjection     : TProjection;
    fHWnd           : HWnd;
    fDomain         : String;
    fOnDomainStatus : TOnDomainStatus;
    NetDialog       : TNetDialog;
  protected
    procedure CompWndProc(var Message: TMessage);
    procedure WMDomainStatus(var Message: TMessage); message WM_DomainStatus;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property Domain     : String read fDomain write fDomain;
    property Projection : TProjection read fProjection write fProjection;
    property ZoomFactor : TZoomFactor read fZoomFactor write fZoomFactor;

    property OnDomainStatus : TOnDomainStatus read fOnDomainStatus write fOnDomainStatus;
  end;

const
  errConnect  = 1;
  errInfo     = 2;
  errMap      = 3;
  errDownload = 4;

  Errors : Array[1..4] of String =
                 ('Cannot connect to database server',
                  'Domain information error',
                  'Domain map error',
                  'Map download error');

procedure Register;

implementation

{$R *.DFM}

uses
  GifImage;

const
  LatLonChars = '-0123456789.';
  CgiURL = 'http://cello.cs.uiuc.edu/cgi-bin/slamm/ip2ll';
  MapURL = 'http://mapweb.parc.xerox.com/map/color=1/border=1/';

// TNetDialog ------------------------------------------------------------------

constructor TNetDialog.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  GifFName := LowerCase(ChangeFileExt(Application.ExeName, '.gif'));
  TempFName := LowerCase(ChangeFileExt(Application.ExeName, '.tmp'));
  PostResults := TStringList.Create;
end;

destructor TNetDialog.Destroy;

begin
  PostResults.Free;
  Inherited Destroy;
end;

procedure TNetDialog.FormShow(Sender: TObject);

begin
  Caption := '';
  statBar.SimpleText := '';
end;

function TNetDialog.GetURL(Source, Dest: string): Boolean;
begin
  try
    Result := (UrlDownloadToFile(nil, PChar(source), PChar(Dest), 0, nil) = 0);
  except
    Result := False;
  end;
end;

function TNetDialog.ParseURLLocation(FName : String) : Boolean;

var
  I, J, X     : Integer;
  sLon, sLat,
  sHt, sMark,
  sProj, Temp : String;

begin
  Result := False;
  if FileExists(FName) then
    PostResults.LoadFromFile(FName)
  else
    Exit;
  with PostResults do
  begin
    J := 0;
    for I := 0 to count - 1 do
      if Pos('<pre>', Strings[I]) > 0 then
      begin
        J := I;
        break;
      end;
    if J = 0 then
    begin
      OnError(errInfo);
      Exit;
    end;
    Temp := Strings[J];
    X := 1;
    sLat := '';
    sLon := '';
    while (X <= Length(Temp)) and (Pos(Temp[X], LatLonChars) = 0) do
      Inc(X);
    while (X <= Length(Temp)) and (Pos(Temp[X], LatLonChars) > 0) do
    begin
      sLat := sLat + Temp[X];
      Inc(X);
    end;
    while (X <= Length(Temp)) and (Pos(Temp[X], LatLonChars) = 0) do
      Inc(X);
    while (X <= Length(Temp)) and (Pos(Temp[X], LatLonChars) > 0) do
    begin
      sLon := sLon + Temp[X];
      Inc(X);
    end;
    J := 0;
    for I := 0 to count - 1 do
      if Pos('</pre>', Strings[I]) > 0 then
      begin
        J := I;
        break;
      end;
    if J = 0 then
    begin
      OnError(errInfo);
      Exit;
    end;
    Temp := Strings[J];
    System.Delete(Temp, 1, 6);
    X := Pos('<', Temp);
    System.Delete(Temp, X, 4);
    System.Insert(',  ', Temp, X);
    X := Pos('<', Temp);
    System.Delete(Temp, X, 4);
    statBar.SimpleText := 'City, State/Country:  ' + Temp;
    case aZoomFactor of
       a15Degrees : sHt := '15';
       a30Degrees : sHt := '30';
       a60Degrees : sHt := '60';
      a120Degrees : sHt := '120';
    end;
    sHt := 'ht=' + sHt + '/';
    case aProjection of
      Rectangular : sProj := 'proj=rect/';
       Elliptical : sProj := '';
    end;
    sMark := 'mark=' + sLat + ',' + sLon + '/';
    sLat := 'lat=' + sLat + '/';
    sLon := 'lon=' + sLon + '/';
    MapRequest := MapURL + sLat + sLon + sMark + sHt + sProj + 'format=.gif';
    Result := True;
  end;
end;

procedure TNetDialog.OnError(Error : Integer);

begin
  statBar.SimpleText := Errors[Error];
  PostMessage(CompHandle, WM_DomainStatus, Error, 0);
end;

procedure TNetDialog.GetInfo;

begin
  Screen.Cursor := crHourGlass;
  with imgMap do
  begin
    Picture := Nil;
    Update;
  End;
  Caption := 'Geographic location for domain:  "' + aDomain + '"';
  statBar.SimpleText := 'Connecting to: ' + CgiURL + ',  please wait...';
  Application.ProcessMessages;
  if GetURL(CgiURL + '?name=' + aDomain, TempFName) then
    if ParseURLLocation(TempFName) then
      begin
        if GetURL(MapRequest, GifFName) then
          begin
            imgMap.Picture.LoadFromFile(GifFName);
            PostMessage(CompHandle, WM_DomainStatus, 0, 0);
          end
        else
          OnError(errDownload);
      end
    else
  else
    OnError(errConnect);
  Screen.Cursor := crDefault;
end;

// TGeoWhois ------------------------------------------------------------------

constructor TGeoWhois.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  fHWnd := AllocateHWnd(CompWndProc);
  NetDialog := TNetDialog.Create(Self);
  NetDialog.Visible := False;
end;

destructor TGeoWhois.Destroy;

begin
  DeallocateHWnd(fHWnd);
  NetDialog.Free;
  Inherited Destroy;
end;

procedure TGeoWhois.CompWndProc(var Message: TMessage);

begin
  Dispatch(Message);
end;

procedure TGeoWhois.WMDomainStatus(var Message: TMessage);

begin
  if Assigned(fOnDomainStatus) then
    fOnDomainStatus(Self, Message.WParam, fDomain);
end;

procedure TGeoWhois.Execute;

begin
  with NetDialog do
  begin
    aZoomFactor := fZoomFactor;
    aProjection := fProjection;
    CompHandle := fHWnd;
    aDomain := fDomain;
    Show;
    GetInfo;
  end;
end;


procedure Register;
begin
  RegisterComponents('LGM', [TGeoWhois]);
end;

end.
